package com.sefl.seflRateQuote.rateQuoteWS;
import java.net.HttpURLConnection;
import sun.misc.BASE64Encoder;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * Helper Class for POST and GET methods using basic http authorization.  In this
 * example we use Sun's BASE64Encoder though any BASE64 encoder should work.
 */

public final class RestfulQuoteHelper {
    private OutputStream outputStream = null;
    private String address = null;
    private Map<String, String> postParameters = null;    
    private String quoteUri = null;
    private String authStringEnc = null;
    
    public RestfulQuoteHelper() {        
    }
    
    /**
     *
     * @param url {@link URL} server URL to submit request
     * @param parameters {@link HashMap} containing request parameters to send
     * @param outputStream {@link OutputStream} in which the results from request will be written to.'
     * @param username 
     * @param password
     * <br/>The output data is written to the outputStream exactly as it is received.
     */
    public RestfulQuoteHelper(final String address, final Map<String, String> parameters, final OutputStream outputStream, final String username, final String password) {
        this.outputStream = outputStream;
        this.address = address;
        this.postParameters = parameters;
        String authString = username + ":" + password;
        this.authStringEnc = new BASE64Encoder().encode(authString.getBytes());
    }
    
    /** Gets the specified URI.
    * @return A string representing the URI
    */
    public String getQuoteURI() {
        return this.quoteUri;
    }
    
    private boolean checkData() {
        return !(outputStream == null || address == null);
    }
    
    /**
     * Perform POST sending postParameters as form data and authenticating 
     * using basic http authorization via the http header. Output stream response
     * can either be accepted as application/json or application/xml.
     * @exception  illegalArgumentException if output stream or URI have not been specified.
     */
    public void doPost() throws IllegalArgumentException {
        if (!checkData()) {
            throw new IllegalArgumentException("Must specify an OutputStream and URL.");
        }
        HttpURLConnection urlConn = null;
        DataInputStream dataRetrieved = null;
        DataOutputStream dataToSend = null;
        try {
            URL url = new URL(address);
            urlConn = (HttpURLConnection) url.openConnection();
            urlConn.setDoInput(true);
            urlConn.setDoOutput(true);
            urlConn.setUseCaches(false);
            urlConn.setRequestMethod("POST");
            urlConn.setRequestProperty("Authorization", "Basic " + authStringEnc);
            urlConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            /**
             * use following if XML response is preferred
             * urlConn.setRequestProperty("Accept", "application/xml");
             */            
            urlConn.setRequestProperty("Accept", "application/json");            
            dataToSend = new DataOutputStream(urlConn.getOutputStream());
            if (postParameters != null && postParameters.size() > 0) {
                StringBuilder content = null;
                for (String key : postParameters.keySet()) {
                    if (postParameters.get(key) != null) {
                        if (content == null) {
                            content = new StringBuilder();
                        } else {
                            content.append("&");
                        }
                        content.append(key).append("=").append(URLEncoder.encode(postParameters.get(key), "UTF-8"));
                    }
                }
                dataToSend.writeBytes(content.toString());
            }
            dataToSend.flush();
            dataRetrieved = new DataInputStream(urlConn.getInputStream());
            quoteUri = urlConn.getHeaderField("location");
            int data = 0;
            while ((data = dataRetrieved.read()) != -1) {
                outputStream.write(data);
            }
        } catch (IOException io) {
            io.printStackTrace();
        } finally {
            if (dataRetrieved != null) {
                try {
                    dataRetrieved.close();
                } catch (IOException ignored) {
                }
            }
            if (dataToSend != null) {
                try {
                    dataToSend.close();
                } catch (IOException ignored) {
                }
            }
            if (urlConn != null) {
                urlConn = null;
            }
        }
    }

     /**
     * Perform GET authenticating using basic http authorization via the http 
     * header. Output stream response can either be accepted as application/json
     * or application/xml.
     * @exception  illegalArgumentException if output stream or URI have not been specified.
     */
    public void doGet() throws IllegalArgumentException {
        if (!checkData()) {
            throw new IllegalArgumentException("Must specify an OutputStream and URL.");
        }
        HttpURLConnection urlConn = null;
        DataInputStream dataRetrieved = null;
        try {
            URL url = new URL(address);
            urlConn = (HttpURLConnection) url.openConnection();
            urlConn.setDoInput(true);
            urlConn.setUseCaches(false);
            urlConn.setRequestMethod("GET");
            urlConn.setRequestProperty("Authorization", "Basic " + authStringEnc);
            /**
             * use following if XML response is preferred
             * urlConn.setRequestProperty("Accept", "application/xml");
             */            
            urlConn.setRequestProperty("Accept", "application/json");
            
            dataRetrieved = new DataInputStream(urlConn.getInputStream());
            int data = 0;
            while ((data = dataRetrieved.read()) != -1) {
                outputStream.write(data);
            }
        } catch (IOException io) {
            io.printStackTrace();
        } finally {
            if (dataRetrieved != null) {
                try {
                    dataRetrieved.close();
                } catch (IOException ignored) {
                }
            }
            if (urlConn != null) {
                urlConn = null;
            }
        }
    }


}
