package com.sefl.seflRateQuote.rateQuoteWS;
import org.json.simple.JSONObject;
import org.json.simple.JSONArray;
import java.util.Iterator;
import org.json.simple.parser.JSONParser;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;
import java.io.ByteArrayOutputStream;
import org.apache.commons.lang.StringEscapeUtils;

/*
 * Example utilizing RestfulQuoteHelper class to connect to the MySEFL 
 * RESTful Quote Interface.  The MySEFL RESTful quotes allow you to submit a
 * new quote via POST method, retreive a summary of that quote or detail on that
 * quote via GET method and finaly retrieve a list of recent quotes submitted
 * via GET method.  All of these functions are demnonstrated in the main  
 * below.
 */

public class RestfulQuoteWSExample {

    private static String reqDetailURI = null;    
 //enter your username here...
    private static String username = "****";
 //enter your password here...   
    private static String password = "****";

    public static void main(String[] args) {
        //submit quote
        String reqURI = submitQuote();
        if (reqURI != null) {
            //retrieve quote summary
            retreiveQuote(reqURI);
            //retrieve quote detail
            retreiveDetailQuote(reqDetailURI);
        }
        //list recent quotes
        listQuote();
    }
    
    /**
     * Submit a new quote.  Build a hashmap of required parameters which will be submitted via POST as
     * form data.  On a successful submit the URI to retrieve a quote summary and detail will both be 
     * returned as well as a quote reference number.  In the example below are are expecting JSON however 
     * this can be changed to XML in the RestFulQuoteHelper Class.
     */    
    public static String submitQuote() {
        String returnData = null;
        JSONParser parser = new JSONParser();
        String quoteReqURI = null;
        try {
            String url = "https://www.sefl.com/webconnect/ratequotes/rest/submitQuote";
            Map<String, String> postParameters = new HashMap<String, String>();
            postParameters.put("CustomerAccount", "001184274");
            postParameters.put("CustomerName", "Demo");
            postParameters.put("CustomerCity", "Columbia");
            postParameters.put("CustomerStreet", "123 Demo St");
            postParameters.put("CustomerState", "SC");
            postParameters.put("CustomerZip", "29201");
            postParameters.put("PickupMonth", "10");
            postParameters.put("PickupDay", "15");
            postParameters.put("PickupYear", "2016");
            postParameters.put("returnX", "Y");
            postParameters.put("rateXML", "Y");
            postParameters.put("Option", "S");
            postParameters.put("Terms", "P");
            postParameters.put("EmailAddress", "test@sefl.com");
            postParameters.put("OriginZip", "29201");
            postParameters.put("OriginCity", "Columbia");
            postParameters.put("OriginState", "SC");
            postParameters.put("OrigCountry", "U");
            postParameters.put("DestinationZip", "23838");
            postParameters.put("DestinationCity", "Chesterfield");
            postParameters.put("DestinationState", "VA");
            postParameters.put("DestCountry", "U");
            postParameters.put("Class1", "100");
            postParameters.put("Weight1", "2856");
            postParameters.put("Description", "Test commodity");
            postParameters.put("LGD", "on");
            OutputStream outputStream = new ByteArrayOutputStream();
            RestfulQuoteHelper http = new RestfulQuoteHelper(url, postParameters, outputStream, username, password);
            http.doPost();
            returnData = outputStream.toString();
            if (returnData != null) {
                JSONObject jsonObject = (JSONObject) parser.parse(returnData.toString());
                quoteReqURI = http.getQuoteURI();
                System.out.println("boolean error:    " + (String) jsonObject.get("errorOccured"));
                System.out.println("error msg:        " + (String) jsonObject.get("errorMessage"));
                System.out.println("quote number:     " + (String) jsonObject.get("quoteNumber"));
                System.out.println("quote uri:        " + StringEscapeUtils.unescapeJava((String) jsonObject.get("location")));
                System.out.println("detail quote uri: " + StringEscapeUtils.unescapeJava((String) jsonObject.get("detailQuoteLocation")));     
                reqDetailURI = StringEscapeUtils.unescapeJava((String) jsonObject.get("detailQuoteLocation"));
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return quoteReqURI;
    }

    
   /**
     *
     * @param reqURI String value of URI to retrieve quote
     * <br/>Retrieve quote summary from URI returned upon submit.  In this example we are parsing
     * JSON however this could be changed to XML in the RestfulQuoteHelper class.
     */    
    public static void retreiveQuote(String reqURI) {
        String returnData = null;
        JSONParser parser = null;
        boolean keepTrying = true;
        boolean timeout = false;
        int tryCount = 0;
        JSONObject jsonObject = null;
        String status = null;
        try {
            OutputStream outputStream = new ByteArrayOutputStream();
            RestfulQuoteHelper http = null;
            while (keepTrying && !timeout) {
                outputStream = new ByteArrayOutputStream();
                http = new RestfulQuoteHelper(reqURI, null, outputStream, username, password);
                http.doGet();
                returnData = outputStream.toString();
                if (returnData != null) {
                    parser = new JSONParser();
                    jsonObject = (JSONObject) parser.parse(returnData.toString());
                    status = (String) jsonObject.get("status");
                    System.out.println(status);
                    if (status.trim().equals("RAT") || status.trim().equals("UTR")) {
                        keepTrying = false;
                        System.out.println("boolean error: " + (String) jsonObject.get("errorOccured"));
                        System.out.println("error msg:     " + (String) jsonObject.get("errorMessage"));
                        System.out.println("quote number:  " + (String) jsonObject.get("quoteNumber"));
                        System.out.println("quote amt:     " + (String) jsonObject.get("rateQuote"));
                        System.out.println("disc perc:     " + (String) jsonObject.get("discountPercent"));
                        System.out.println("disc amt:      " + (String) jsonObject.get("discountAmount"));
                        System.out.println("quote status:  " + (String) jsonObject.get("status"));
                    } else {
                        tryCount++;
                        Thread.sleep(1000);
                        if (tryCount > 120) {
                            timeout = true;
                        }
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

   /**
     *
     * @param reqURI String value of URI to retrieve quote detail
     * <br/>Retrieve quote detail from URI returned upon submit.  In this example we are parsing
     * JSON however this could be changed to XML in the RestfulQuoteHelper class.
     */        
    public static void retreiveDetailQuote(String reqDtlURI) {
        String returnData = null;
        JSONParser parser = null;
        boolean keepTrying = true;
        boolean timeout = false;
        int tryCount = 0;
        JSONObject jsonObject = null;
        String status = null;
        try {
            OutputStream outputStream = new ByteArrayOutputStream();
            RestfulQuoteHelper http = null;
            while (keepTrying && !timeout) {
                outputStream = new ByteArrayOutputStream();
                http = new RestfulQuoteHelper(reqDtlURI, null, outputStream, username, password);
                http.doGet();
                returnData = outputStream.toString();
                if (returnData != null) {
                    parser = new JSONParser();
                    jsonObject = (JSONObject) parser.parse(returnData.toString());
                    status = (String) jsonObject.get("status");
                    System.out.println(status);
                    if (status.trim().equals("RAT") || status.trim().equals("UTR")) {
                        keepTrying = false;
                        System.out.println("boolean error:     " + (String) jsonObject.get("errorOccured"));
                        System.out.println("error msg:         " + (String) jsonObject.get("errorMessage"));
                        System.out.println("quote nunmber:     " + (String) jsonObject.get("quoteNumber"));
                        System.out.println("pickup date:       " + (String) jsonObject.get("pickupDate"));
                        System.out.println("cust acct:         " + (String) jsonObject.get("custAccount"));
                        System.out.println("cust name:         " + (String) jsonObject.get("custName"));
                        System.out.println("cust addr:         " + (String) jsonObject.get("custAddress"));
                        System.out.println("cust addr2:        " + (String) jsonObject.get("custAddress2"));
                        System.out.println("cust city:         " + (String) jsonObject.get("custCity"));
                        System.out.println("cust state:        " + (String) jsonObject.get("custState"));
                        System.out.println("cust zip:          " + (String) jsonObject.get("custZip"));
                        System.out.println("orig city:         " + (String) jsonObject.get("origCity"));
                        System.out.println("orig state:        " + (String) jsonObject.get("origState"));
                        System.out.println("orig zip:          " + (String) jsonObject.get("origZip"));
                        System.out.println("orig DIP:          " + (String) jsonObject.get("origDIP"));
                        System.out.println("dest city:         " + (String) jsonObject.get("destCity"));
                        System.out.println("dest state:        " + (String) jsonObject.get("destState"));
                        System.out.println("dest zip:          " + (String) jsonObject.get("destZip"));
                        System.out.println("dest DIP:          " + (String) jsonObject.get("destDIP"));
                        System.out.println("quote DIP:         " + (String) jsonObject.get("quoteDIP"));
                        System.out.println("transit miles:     " + (String) jsonObject.get("transitMiles"));
                        System.out.println("transit time:      " + (String) jsonObject.get("transitTime"));
                        System.out.println("spec instr:        " + (String) jsonObject.get("specInstr"));
                        System.out.println("orig phone:        " + (String) jsonObject.get("origPhone"));
                        System.out.println("dest phone:        " + (String) jsonObject.get("destPhone"));
                        System.out.println("rate quote:        " + (String) jsonObject.get("rateQuote"));
                        System.out.println("disc perc:         " + (String) jsonObject.get("discountPercent"));
                        System.out.println("disc amt:          " + (String) jsonObject.get("discountAmount"));
                        System.out.println("status:            " + (String) jsonObject.get("status"));
                        // loop detail lines
                        JSONArray dtls = (JSONArray) jsonObject.get("details");
                        Iterator<JSONObject> iterator = dtls.iterator();
                        while (iterator.hasNext()) {
                            JSONObject dtlObj = (JSONObject) iterator.next();
                            System.out.println("type charge:       " + (String) dtlObj.get("typeCharge"));
                            System.out.println("description:       " + (String) dtlObj.get("description"));
                            System.out.println("weight:            " + (String) dtlObj.get("weight"));
                            System.out.println("class:             " + (String) dtlObj.get("ratingClass"));
                            System.out.println("rate:              " + (String) dtlObj.get("rate"));
                            System.out.println("charges:           " + (String) dtlObj.get("charges"));
                        }
                    } else {
                        tryCount++;
                        Thread.sleep(1000);
                        if (tryCount > 120) {
                            timeout = true;
                        }
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     *
     * <br/>Retrieve list of recently submitted quotes.  In this example we are parsing
     * JSON however this could be changed to XML in the RestfulQuoteHelper class.
     */    
    public static String listQuote() {
        String returnData = null;
        JSONParser parser = new JSONParser();
        JSONObject jsonObject = null;
        try {
            String url = "https://www.sefl.com/webconnect/ratequotes/rest/list";
            OutputStream outputStream = new ByteArrayOutputStream();
            RestfulQuoteHelper http = new RestfulQuoteHelper(url, null, outputStream, username, password);
            http.doGet();
            returnData = outputStream.toString();
            if (returnData != null) {
                parser = new JSONParser();
                jsonObject = (JSONObject) parser.parse(returnData.toString());
                System.out.println("boolean error:     " + (String) jsonObject.get("errorOccured"));
                System.out.println("boolean msg:       " + (String) jsonObject.get("errorMessage"));
                // loop quotes
                JSONArray quotes = (JSONArray) jsonObject.get("quotes");
                Iterator<JSONObject> iterator = quotes.iterator();
                while (iterator.hasNext()) {
                    JSONObject quoteObj = (JSONObject) iterator.next();
                    System.out.println("quote number:      " + (String) quoteObj.get("quoteNumber"));
                    System.out.println("cust acct:         " + (String) quoteObj.get("custAccount"));
                    System.out.println("cust name:         " + (String) quoteObj.get("custName"));
                    System.out.println("cust city:         " + (String) quoteObj.get("custCity"));
                    System.out.println("cust state:        " + (String) quoteObj.get("custState"));
                    System.out.println("cust zip:          " + (String) quoteObj.get("custZip"));
                    System.out.println("orig city:         " + (String) quoteObj.get("origCity"));
                    System.out.println("orig state:        " + (String) quoteObj.get("origState"));
                    System.out.println("orig zip:          " + (String) quoteObj.get("origZip"));
                    System.out.println("dest city:         " + (String) quoteObj.get("destCity"));
                    System.out.println("dest state:        " + (String) quoteObj.get("destState"));
                    System.out.println("dest zip:          " + (String) quoteObj.get("destZip"));
                    System.out.println("weight:            " + (String) quoteObj.get("weight"));
                    System.out.println("status:            " + (String) quoteObj.get("status"));
                    System.out.println("terms:             " + (String) quoteObj.get("terms"));
                    System.out.println("rate quote:        " + (String) quoteObj.get("rateQuote"));
                    System.out.println("pickup date:       " + StringEscapeUtils.unescapeJava((String) quoteObj.get("pickupDate")));
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return returnData;
    }

}
